import{p as B,a as t,f as h,h as l,e,i as E,t as F,l as G,m as r,n as _,r as I,x as J}from"./index-DUEdOLeb.js";import b from"./Markdown-E0saaMx0.js";import"./purify.es-HRjpPm7y.js";import"./katex-XbL3y5x-.js";/* empty css                  */var K=h("<details><summary>Details</summary> <!></details>"),N=h('<div><div class="confirm-header"><h3 class="svelte-1dszvfa"> </h3></div> <div class="confirm-body svelte-1dszvfa"><!> <!></div> <div class="confirm-actions svelte-1dszvfa"><button class="btn-cancel svelte-1dszvfa"> </button> <button class="btn-confirm svelte-1dszvfa"> </button></div></div>');function U(g,a){B(a,!0);let x=t(a,"title",3,"Confirm"),z=t(a,"message",3,""),C=t(a,"confirmLabel",3,"Confirm"),k=t(a,"cancelLabel",3,"Cancel"),L=t(a,"variant",3,"info"),d=t(a,"details",3,void 0),o=t(a,"onResult",3,void 0);function p(){o()?.({confirmed:!0})}function w(){o()?.({confirmed:!1})}var n=N(),v=e(n),y=e(v),H=e(y),c=l(v,2),f=e(c);b(f,{get content(){return z()},allowHtml:!1,padded:!1});var R=l(f,2);{var D=s=>{var u=K(),A=l(e(u),2);b(A,{get content(){return d()},allowHtml:!1,padded:!1}),_(s,u)};E(R,s=>{d()&&s(D)})}var M=l(c,2),i=e(M);i.__click=w;var j=e(i),m=l(i,2);m.__click=p;var q=e(m);F(()=>{G(n,1,`confirm ${L()}`,"svelte-1dszvfa"),r(H,x()),r(j,k()),r(q,C())}),_(g,n),I()}J(["click"]);export{U as default};
