import{p as B,a as u,u as E,d as T,s as F,f as p,j as H,i as b,g as a,e as d,h as j,t as g,l as w,m as y,n as o,o as I,c as A,r as J,G as K,D as L,A as O,x as P}from"./index-DUEdOLeb.js";import Q from"./Markdown-E0saaMx0.js";import"./purify.es-HRjpPm7y.js";import"./katex-XbL3y5x-.js";/* empty css                  */var S=p('<span class="icon"> </span>'),U=p("<button><!> </button>"),V=p('<div class="empty">No content.</div>'),W=p('<div><div class="tab-list svelte-1f2pv7a"></div> <div class="tab-panel svelte-1f2pv7a"><!></div></div>');function ta(C,n){B(n,!0);let h=u(n,"tabs",19,()=>[]),D=u(n,"defaultTab",3,0),G=u(n,"variant",3,"line"),i=F(0);E(()=>{T(i,D())});const v=A(()=>h()[a(i)]);var f=W(),x=d(f);H(x,21,h,I,(e,t,l)=>{var r=U();r.__click=()=>T(i,l,!0);var c=d(r);{var s=m=>{var k=S(),z=d(k);g(()=>y(z,a(t).icon)),o(m,k)};b(c,m=>{a(t).icon&&m(s)})}var _=j(c);g(()=>{w(r,1,`tab ${l===a(i)?"active":""}`,"svelte-1f2pv7a"),r.disabled=a(t).disabled,y(_,` ${a(t).label??""}`)}),o(e,r)});var M=j(x,2),N=d(M);{var R=e=>{{let t=A(()=>a(v).props??{});K(e,{get component(){return a(v).component},get props(){return a(t)}})}},q=e=>{var t=L(),l=O(t);{var r=s=>{Q(s,{get content(){return a(v).content},padded:!1})},c=s=>{var _=V();o(s,_)};b(l,s=>{a(v)?.content?s(r):s(c,!1)},!0)}o(e,t)};b(N,e=>{a(v)?.component?e(R):e(q,!1)})}g(()=>w(f,1,`tabs ${G()}`,"svelte-1f2pv7a")),o(C,f),J()}P(["click"]);export{ta as default};
